/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher for MPS2
 * Note(s): possible defines set in "options for target - C/C++ - Define"
 *            __USE_LCD    - enable Output on GLCD
 *            __USE_TIMER0 - use Timer0  to generate timer interrupt
 *                         - use SysTick to generate timer interrupt (default)
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2013 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include <stdio.h>
#include "Device.h"                     // Keil::Board Support:V2M-MPS2:Common
#include "GLCD.h"                       // Keil::Board Support:V2M-MPS2:Graphic LCD
#include "LED.h"                        // Keil::Board Support:V2M-MPS2:LED
#include "Keyboard.h"                   // Keil::Board Support:V2M-MPS2:Keyboard
#include "Serial.h"                     // Keil::Board Support:V2M-MPS2:Serial

#define __FI        1                   /* Font index 16x24                   */

uint32_t LEDOn, LEDOff;
uint32_t ticks_1s;                      /* Flag activated each second         */


/*----------------------------------------------------------------------------
  SysTick / Timer0 IRQ Handler
 *----------------------------------------------------------------------------*/
#ifdef __USE_TIMER0
void TIMER0_Handler  (void) {
#else
void SysTick_Handler (void) {
#endif
  static uint32_t ticks;
  static uint32_t ticks_10ms;

  if (ticks_10ms++ >= 99) {             /* Set Clock1s to 1 every second      */
    ticks_10ms = 0;
    ticks_1s   = 1;
  }

  switch (ticks++) {
    case   0: LEDOn  = 1; break;
    case  40: LEDOff = 1; break;
    case  45: ticks  = 0;break;

    default:
      if (ticks > 50) {
        ticks = 0;
      }
  }

#ifdef __USE_TIMER0
  CMSDK_TIMER0->INTCLEAR = (1ul <<  0);    /* clear interrupt                 */
#endif
}


#ifdef __USE_TIMER0
/*----------------------------------------------------------------------------
  Timer0 Configuration
 *----------------------------------------------------------------------------*/
void Timer0_Config (uint32_t ticks) {

  CMSDK_TIMER0->INTCLEAR =  (1ul <<  0);   /* clear interrupt                 */
  CMSDK_TIMER0->RELOAD   =  (ticks - 1);   /* set reload value                */
  CMSDK_TIMER0->CTRL     = ((1ul <<  3) |  /* enable Timer interrupt          */
                            (1ul <<  0) ); /* enable Timer                    */

  NVIC_EnableIRQ(TIMER0_IRQn);             /* Enable interrupt in NVIC        */
}
#endif


/*----------------------------------------------------------------------------
  Main function
 *----------------------------------------------------------------------------*/
int main (void) {
  uint32_t ledMax = LED_NumLeds() - 1;
  uint32_t ledNum = 0;
  uint32_t keyMax = Keyboard_NumKeys();
  uint32_t keyNum = 0;
  int32_t dir = 1;
  int32_t keyboard = 0, keyboard_ = -1;
  char text[] = "Hello World";

  SystemCoreClockUpdate();

  LED_Initialize();                         /* LED Initialization             */
  Keyboard_Initialize();
  Serial_Initialize();
#ifdef __USE_LCD
  GLCD_Initialize();                        /* Initialize the GLCD            */

  /* display initial screen */
  GLCD_Clear(White);                        /* Clear the GLCD                 */
  GLCD_SetBackColor(Blue);
  GLCD_SetTextColor(Red);
  GLCD_DisplayString(0, 0, __FI, "   V2M-MPS2 Demo    ");
  GLCD_DisplayString(1, 0, __FI, "      Blinky        ");
  GLCD_DisplayString(2, 0, __FI, "   www.keil.com     ");

  GLCD_SetBackColor(White);
  GLCD_SetTextColor(Black);
  switch ((SCB->CPUID >> 4) & 0xFFF) {
    case 0xC20:
      GLCD_DisplayString(4, 0, __FI, "   ARM Cortex-M0    ");
      break;
    case 0xC60:
      GLCD_DisplayString(4, 0, __FI, "   ARM Cortex-M0+   ");
      break;
    case 0xC21:
      GLCD_DisplayString(4, 0, __FI, "   ARM Cortex-M1    ");
      break;
    case 0xC23:
      GLCD_DisplayString(4, 0, __FI, "   ARM Cortex-M3    ");
      break;
    case 0xC24:
      GLCD_DisplayString(4, 0, __FI, "   ARM Cortex-M4    ");
      break;
    default:
      GLCD_DisplayString(4, 0, __FI, "  unknown Cortex-M  ");
      break;
  }
  GLCD_DisplayString(6, 2, __FI, "LED: ");
  GLCD_DisplayString(7, 2, __FI, "KEY: ");
#endif

#ifdef __USE_TIMER0
  Timer0_Config (SystemCoreClock / 100);    /* Generate interrupt each 10 ms  */
#else
  SysTick_Config(SystemCoreClock / 100);    /* Generate interrupt each 10 ms  */
#endif

  while (1) {
    if (LEDOn) {
      LEDOn = 0;
      LED_On (ledNum);                      /* Turn specified LED on          */
#ifdef __USE_LCD
      GLCD_SetTextColor(Green);
      GLCD_DisplayChar(6, 7+ledNum, __FI, 0x80+ 1);
#endif
    }

    if (LEDOff) {
      LEDOff = 0;
      LED_Off (ledNum);                     /* Turn specified LED off         */
#ifdef __USE_LCD
      GLCD_SetTextColor(LightGrey);
      GLCD_DisplayChar(6, 7+ledNum, __FI, 0x80+ 0);
#endif

      ledNum += dir;                        /* Change LED number              */

      if (dir == 1 && ledNum == ledMax) {
        dir = -1;                           /* Change direction to down       */
      }
      else if (ledNum == 0) {
        dir =  1;                           /* Change direction to up         */
      }
    }

		keyboard = Keyboard_GetKeys();

		if (keyboard ^ keyboard_) {
      keyboard_ = keyboard;

      for (keyNum = 0; keyNum < keyMax; keyNum++) {
        if (keyboard & (1ul << keyNum)) {
#ifdef __USE_LCD
          GLCD_SetTextColor(Black);
          GLCD_DisplayChar(7, 7+keyNum, __FI, 0x80+ 1);
#endif
        } else {
#ifdef __USE_LCD
          GLCD_SetTextColor(LightGrey);
          GLCD_DisplayChar(7, 7+keyNum, __FI, 0x80+ 0);
#endif
        }
      }

		}

    /* Print message with AD value every second                               */
    if (ticks_1s) {
      ticks_1s = 0;

      printf("%s\r\n", text);
    }
  }
}
